{% load staticfiles %}
{% if next_council_meeting %}
    <h3><i class="fa fa-fw fa-institution"></i> Next City Council Meeting</h3>
    <div class="panel panel-default">
        <div class="panel-body">
            <div class='row'>
                <div class='col-sm-4'>
                    <a href='{{next_council_meeting.event_page_url}}'><img class='img-responsive img-thumbnail' src='{% static 'images/city-council.jpg' %}' /></a>
                </div>
                <div class='col-sm-8'>
                    <h4><a href="{{next_council_meeting.event_page_url}}">{{next_council_meeting.name}}</a></h4>
                    <p>{{next_council_meeting.description}}</p>

                    <p class="small text-muted">
                        <i class="fa fa-fw fa-calendar-o"></i> {{next_council_meeting.start_time | date:"D n/d/Y"}}<br/>
                        <i class="fa fa-fw fa-clock-o"></i> {{next_council_meeting.start_time | date:"g:i a"}}<br/>
                        <i class="fa fa-fw fa-map-marker"></i> {{next_council_meeting.location_name}}<br />
                    </p>
                    <a class='btn btn-primary btn-sm' href="{{next_council_meeting.event_page_url}}">
                        Details
                        <i class="fa fa-fw fa-chevron-right"></i>
                    </a>

                </div>
            </div>
        </div>
    </div>
{% endif %}

{% if upcoming_committee_meetings %}
    <h3>
        <span class="non-mobile-only"><i class="fa fa-fw fa-group"></i> </span>
        Upcoming Committee Meetings
    </h3>
    {% for event in upcoming_committee_meetings %}
        {% include "partials/event_item.html" %}
    {% endfor %}
    <br/>
    <a href="/events/" class="btn btn-sm btn-primary">
        View All Upcoming Events
        <i class="fa fa-fw fa-chevron-right"></i>
    </a>
{% endif %}
